<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

use Ark\Media\Helper;

defined( '_JEXEC' ) or die;

class PlgTemplateXTDArkMedia extends JPlugin
{
	/**
	 * @var		object	Application object
	 */
	public $app;

	/**
	 * @var		string	Ark Extension Name
	 */
	protected $name		= 'com_arkmedia';

	/**
	 * @var		bool	Load the Language File on Instantiation.
	 */
	protected $autoloadLanguage = true;

	/**
	 * Display An Ark Extension Button.
	 *
	 * @param	string	$name	The Name of the Template (e.g. ark_bounce).
	 * @param	string	$asset	The Name of the Asset Being Edited.
	 * @param	int		$author	The ID of the Current Author/User.
	 *
	 * @return	object	A Two Element object of (button, menu items) or false if not Authorised.
	 */
	public function onDisplay( $name, $asset, $author )
	{
		$user 		= JFactory::getUser();
		$extension 	= $this->app->input->get( 'option' );

		if( 
			$user->authorise( 'core.edit', $asset )
			|| $user->authorise( 'core.create', $asset )
			|| ( count( $user->getAuthorisedCategories( $asset, 'core.create' ) ) > 0 )
			|| ($user->authorise( 'core.edit.own', $asset ) && $author == $user->id )
			|| ( count( $user->getAuthorisedCategories( $extension, 'core.edit') ) > 0 )
			|| ( count( $user->getAuthorisedCategories( $extension, 'core.edit.own') ) > 0 && $author == $user->id )
		)
		{
			$loader	= JPATH_ADMINISTRATOR . chr( 47 ) . 'components' . chr( 47 ) . $this->name . chr( 47 ) . 'initiate.php';

			// Load Ark
			if( JFile::exists( $loader ) )
			{
				// Initiate Ark
				defined( '_ARKMEDIA_EXEC' ) or define( '_ARKMEDIA_EXEC', true );

				require_once $loader;

				// Load Asset Path
				$basepath = str_replace( ARKMEDIA_ROOT . ARKMEDIA_DS, '', dirname( __FILE__ ) . ARKMEDIA_DS );
				$options 	= (object)array(
								'name' 			=> $this->_name,
								'type' 			=> $this->_type,
								'asset'			=> $asset,
								'author'		=> $author,
								'params' 		=> $this->params, 
								'id' 			=> $this->_name
							);

				// Render Assets
				echo Helper::layout( 'js.xtd', $options, array( 'base' => $basepath . 'layouts', 'client' => 'auto' ) );
			
				$data 					= new stdClass;
				$data->id				= $this->_name;

				// Supply main toolbar button
				$data->button			= new stdClass;
				$data->button->text		= JText::_( 'PLG_TEMPLATE-XTD_' . $this->_name . '_BTN' );
				$data->button->icon		= 'image';

				return $data;
			}//end if
		}//end if

		return false;
	}//end function
}//end class