<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

use Ark\Media\Helper, Ark\Media\HelperStack, Ark\Media\HelperFileSystem;

defined( '_JEXEC' ) or die();

class plgArkMediaFeatureImageEditing extends JPlugin
{
	/**
	 * @var		string	Plugin Name (for namespacing, config retrieval & more!)
	 */
	protected $name = 'imageediting';

	/**
	 * @var		string	Stack Name to Restrict to
	 */
	protected $stack = 'images';

	/**
	 * @var		string	The Root of the Joomla Installation Including Trailing DS
	 */
	protected $base;

	/**
	 * @var		string	The Relative Path to this Plugin
	 */
	protected $plugin;

	/**
	 * @var		string	Base for JText
	 */
	protected $jtext;

	/**
	 * Constructor
	 */
	public function __construct( &$subject, $config )
	{
		// Load Language for Title
		JFactory::getLanguage()->load( 'plg_' . ARKMEDIA_PLUGIN_FEATURE . '_' . $this->name, JPATH_ADMINISTRATOR );

		$this->base 	= ARKMEDIA_ROOT . ARKMEDIA_DS;
		$this->plugin 	= str_replace( $this->base, '', ARKMEDIA_PLUGIN_FEATURES . $this->name . ARKMEDIA_DS );
		$this->jtext 	= 'PLG_' . JString::strtoupper( ARKMEDIA_PLUGIN_FEATURE ) . '_' . JString::strtoupper( $this->name ) . '_';

		parent::__construct( $subject, $config );
	}//end function

	/**
	 * Catch All Ark Form Events to Add Additional JForm/Items to the Forms
	 *
	 * @param	string	$name		The Current Form Name
	 * @param	object	$form		The Form Object to Manipulate
	 * @param	bool	$namespace	Whether to Namespace Each Form Field With the Stack Name (opt in)
	 *
	 * @return	void
	 */
	public function onArkAfterForm( &$name = '', &$form = null, $namespace = false )
	{
		$file = $this->base . $this->plugin . 'forms/' . $name . '.xml';

		// If We Have Our Own Additions Then Add Them
		if( JFile::exists( $file ) )
		{
			$xml = simplexml_load_file( $file );

			// Set Each Plugin Field/Node With a Namespace Attribute (for the loader to know who merged which field)
			// @see	[#2631] For Add to Entire JForm Rather Than Merged Form Only.
			if( $namespace )
			{
				if( $xml instanceof SimpleXMLElement )
				{
					// Add a Namespace Attribute to Each XML Node
					switch( $namespace )
					{
						default :
						case 'field' :
							$nodes = $xml->xpath( '//field' );
							break;

						case 'fieldset' :
							$nodes = $xml->xpath( '//' . $namespace );
							break;
					}//end switch

					if( count( $nodes ) )
					{
						foreach( $nodes as $node )
						{
							$node['data-stack'] = $this->stack;
						}//end foreach
					}//end if
				}//end if
			}//end if

			// Now Merge the Two Forms
			$form->load( $xml );
		}//end if
	}//end function

	/**
	 * Catch Main Ark JS Renderer & Add Our JS/Layout.
	 *
	 * @param	object	$view		The View Object.
	 *
	 * @return	void
	 */
	public function onArkAfterRenderJS( &$view = null )
	{
		$options = (object)array( 'base' => $this->base, 'name' => $this->name, 'plugin' => $this->plugin, 'jtext' => $this->jtext, 'stack' => $this->stack, 'params' => $this->params );
		$layouts = $this->base . $this->plugin . 'layouts';

		echo Helper::layout( 'js.' . $this->name, $options, array( 'base' => $layouts, 'client' => 'auto' ) );
	}//end function

	/**
	 * Catch Controller Task Initiations for Intercepting Custom Tasks.
	 *
	 * @param	object	$ctrl		The Controller Object.
	 *
	 * @method	ajax
	 *
	 * @return	void
	 */
	public function onArkBeforeTask( &$ctrl = null )
	{
		$app 	= JFactory::getApplication();
		$task 	= $app->input->get( 'task', false, 'cmd' );

		// Catch Our Task
		if( $task === $this->name )
		{
			// Ignore Passed Stack ($app->input->get( 'stack', false, 'cmd' )) as an Additional Security Check
			$stack 		= $this->stack;
			$path 		= $app->input->get( 'path', false, 'string' ); // Path Filter Strips Out Spaces...
			$file 		= $app->input->get( 'file', false, 'string' );
			$image 		= $app->input->get( 'image', false, 'string' );
			$data		= (object)array( 'status' => 0, 'message' => array() );
			$model		= $ctrl->getModel( 'list' );
			$override 	= $this->params->get( 'override', true ); // @todo Plumb in this Option

			// Check the Data
			if( $stack && $path && $file && $image )
			{
				try
				{
					// Check Permissions
					if( !Helper::actions( 'ark.action.edit' ) )
					{
						throw new Exception( JText::_( ARKMEDIA_JTEXT . 'MSG_PERMISSION_NOTAUTHORISEDUPLOAD_FAIL' ) );
					}//end if

					// Load Basic Stack From Cache
					HelperStack::load( false, false );

					// Validate Stack & Path
					if( HelperStack::find( $stack, $path, $file ) )
					{
						// Upload File Data
						$filename = HelperFileSystem::uploadBase64( $path, $image, $file, $override );

						// If Success Clear Stack for New Thumbnail & File to be Generated
						if( $filename )
						{
							// Clear Stack & Client Cache
							$model->refresh();

							// Add Success Data
							$data->message[] 	= JText::_( $this->jtext . 'MSG_AJAX_SUCCESS' );
							$data->status 		= 1;
						}//end if
					}
					else
					{
						throw new Exception( JText::sprintf( ARKMEDIA_JTEXT . 'MSG_FILEINVALIDITEM_FAIL', $file ) );
					}//end if
				}
				catch( Exception $e )
				{
					// Add Generic Message First
					$data->message[] = JText::_( $this->jtext . 'MSG_AJAX_FAIL' );

					// Add Specific
					if( $e->getMessage() )
					{
						$data->message[] = $e->getMessage();
					}//end if
				}//end try
			}//end if

			// Collapse Messages
			$data->message = implode( chr( 32 ), $data->message );

			echo json_encode( $data );

			jexit();
		}//end if
	}//end function
}//end class