<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

use Ark\Media\Helper;

defined( '_JEXEC' ) or die;

$name 		= $displayData->name;
$stack 		= $displayData->stack;
$jtext 		= $displayData->jtext;
$params 	= $displayData->params;
$jspath		= $displayData->plugin . 'js' . ARKMEDIA_DS;
$csspath	= $displayData->plugin . 'css' . ARKMEDIA_DS;
$lock		= Helper::html( 'icon.uikit', 'lock' ) . chr( 32 );
$error		= Helper::html( 'icon.uikit', 'warning uk-text-danger' ) . chr( 32 );

// Add CSS
Helper::html( 'ark.css', 'ark-' . $name . '.min', $csspath, array(), false );

// Ark Image Editing Initiator File
Helper::html( 'ark.js', 'ark-' . $name . '.min', $jspath, false, false );
// Fabric Library
// @required plg-imageediting/plg-fabric.min.js
// Root Upload Library File
// @required plg-imageediting/plg-' . $name . '-core.min.js
// Support Undo/Redo History
// @required plg-imageediting/plg-history.min.js
// Support Image Rotating
// @required plg-imageediting/plg-rotate.min.js
// Support Image Cropping
// @required plg-imageediting/plg-crop.min.js
// Support Saving
// @required plg-imageediting/plg-save.min.js
// Support Not Saving
// @required plg-imageediting/plg-nosave.min.js
// Support Extra Rotations
// @required plg-imageediting/plg-rotatemore.min.js
// Support Image Flipping
// @required plg-imageediting/plg-flip.min.js
// Support Crop Customiser (after main crop but before other crop files)
// @required plg-imageediting/plg-cropcustomiser.min.js
// Support Crop Dimensions
// @required plg-imageediting/plg-cropdimensions.min.js
// Support Crop Presets
// @required plg-imageediting/plg-croppresets.min.js
// Support Image Resizing
// @required plg-imageediting/plg-resize.min.js
// Support Resize Dimensions
// @required plg-imageediting/plg-resizedimensions.min.js
// Support Resize Presets
// @required plg-imageediting/plg-resizepresets.min.js
// Support Click Scrolling
// @required plg-imageediting/plg-kinetic.min.js

// Get Presets (filter out blank array entries from saving plugin without custom entries)
$cropper 	= 'crop-dimensions';
$resizer 	= 'resize-dimensions';
$crops 		= array_filter( $params->get( $cropper, array() ) );
$crops 		= json_decode( current( $crops ) );
$resizes 	= array_filter( $params->get( $resizer, array() ) );
$resizes 	= json_decode( current( $resizes ) );
$presets	= (object)array( 'crop' => array(), 'resize' => array() );

// Pivot, Decode & Rename Crop Presets (data arrives as single array of json pivots)
if( $crops )
{
	for( $i = 0; $i < count( $crops->{ $cropper . '-x' } ); $i++ )
	{
		$x = $crops->{ $cropper . '-x' }[$i];
		$y = $crops->{ $cropper . '-y' }[$i];
		$c = $crops->{ $cropper . '-constrain' }[$i];
		$d = $crops->{ $cropper . '-default' }[$i];

		if( $x && $y && is_numeric( $x ) && is_numeric( $y ) )
		{
			$presets->crop[] = (object)array( 'x' => (int)$x, 'y' => (int)$y, 'constrain' => (int)$c, 'primary' => (int)$d );
		}//end if
	}//end for
}//end if

// Pivot, Decode & Rename Resize Presets (data arrives as single array of json pivots)
if( $resizes )
{
	for( $i = 0; $i < count( $resizes->{ $resizer . '-x' } ); $i++ )
	{
		$x = $resizes->{ $resizer . '-x' }[$i];
		$y = $resizes->{ $resizer . '-y' }[$i];
		$p = $resizes->{ $resizer . '-percent' }[$i];
		$u = $resizes->{ $resizer . '-unit' }[$i];
		$d = $resizes->{ $resizer . '-default' }[$i];

		// Check Unit Types
		if(
			( $u === 'px' && ( $x || $y ) && ( is_numeric( $x ) || is_numeric( $y ) ) )
			||
			( $u === 'pc' && $p && is_numeric( $p ) )
		  )
		{
			$presets->resize[] = (object)array( 'x' => (int)$x, 'y' => (int)$y, 'pc' => (int)$p, 'primary' => (int)$d );
		}//end if
	}//end for
}//end if
?>
<script>
	jQuery( document ).ready( function( $ )
	{
		jQuery.fn.<?php echo $name; ?>(
		{
			css			: {
							editactionbar 	: '#<?php echo Helper::css( 'edit.actions' ); ?>',
							preview			: {
												container 	: '#<?php echo Helper::css( 'edit.preview' ); ?>',
												holder 		: '.preview',
												preview 	: 'img'
											  },
							control			: '[data-edit-action]',
							yes 			: { 
												value 		: '<?php echo Helper::css( 'button.button' ) . chr( 32 ) . Helper::css( 'button.type.danger' ); ?>', 
												selector 	: '.<?php echo Helper::css( 'button.button' ) . '.' . Helper::css( 'button.type.danger' ); ?>'
											},
							radius 			: { 
												value 		: '<?php echo Helper::css( 'button.radius.right' ); ?>', 
												selector 	: '.<?php echo Helper::css( 'button.radius.right' ); ?>'
											}
						  },
			html 		: {
							ajax			: '<?php echo JRoute::_( 'index.php?option=' . ARKMEDIA_COMPONENT . '&task=list.' . $name, false ); ?>',
							glue			: '<?php echo JText::_( $jtext . 'GLUE' ); ?>',
							percent			: '<?php echo JText::_( $jtext . 'PERCENT' ); ?>',
							auto			: '<?php echo JText::_( $jtext . 'AUTO' ); ?>',
							icons			: {
												lock		: '<?php echo $lock; ?>',
												error		: '<?php echo $error; ?>'
											  },
							messages		: {
												confirm		: '<?php echo JText::_( $jtext . 'MSG_CONTINUE_CONFIRM', true ); ?>'
											  },
							errors			: {
												stack 		: '<?php echo JText::_( $jtext . 'MSG_STACK_FAIL', true ); ?>',
												noedit 		: '<?php echo JText::_( $jtext . 'MSG_NOEDIT_FAIL', true ); ?>',
												malform 	: '<?php echo JText::_( $jtext . 'MSG_AJAXMALFORM_FAIL', true ); ?>',
												fail 		: '<?php echo JText::_( $jtext . 'MSG_AJAXFAIL_FAIL', true ); ?>',
												ajax 		: '<?php echo JText::_( $jtext . 'MSG_AJAXAJAX_FAIL', true ); ?>',
												resize 		: {
																big 	: '<?php echo JText::_( $jtext . 'MSG_RESIZETOOLARGE_FAIL', true ); ?>',
																small 	: '<?php echo JText::_( $jtext . 'MSG_RESIZETOOSMALL_FAIL', true ); ?>',
																invalid : '<?php echo JText::_( $jtext . 'MSG_RESIZEINVALID_FAIL', true ); ?>'
															  }
											  },
							yes				: '<?php echo JText::_( ARKMEDIA_JTEXT . 'BTN_CONTINUENOSAVE', true ); ?>'
						  },
			canvas		: {
							bg 				: 'rgba(255,255,255,0)', // Set BG to transparent to preserve alpha pngs
							height 			: <?php echo (int)$params->get( 'height', 500 ); ?>,
							handle			: {
												size 	: 12,
												colour 	: '#ffffff',
												border 	: null,
												overlay	: 'rgba( 0, 0, 0, 0.7 )'
											  }
						  },
			crops		: <?php echo json_encode( $presets->crop ); ?>,
			resizes		: <?php echo json_encode( $presets->resize ); ?>,
			scroller	: <?php echo 1; ?>,
			autosave	: <?php echo (int)$params->get( 'auto-save', true ); ?>,
			confirm		: <?php echo (int)$params->get( 'confirm', true ); ?>,
			stack		: '<?php echo $stack; ?>',
			permission	: 'ark.action.edit'
		});
	});
</script>