<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

use Ark\Media\Helper, Ark\Media\HelperStack;

defined( '_JEXEC' ) or die;

$name 		= $displayData->name;
$form 		= $displayData->form;
$stack 		= $displayData->stack;
$params		= $displayData->params;
$fields 	= $displayData->fields;
$settings 	= $displayData->settings;
$app		= JFactory::getApplication();
$callback	= ARKMEDIA_COMPONENT_ID . 'callback' . $name;
$ids		= array();
$paths		= array();
$version  	= new JVersion;

// Add Root Library JS (not the rest of the library)
Helper::html( 'ark.framework', array( 'overrides' => false, 'ark' => false, 'fonts' => false ) );

// Add JS
Helper::html( 'ark.js', 'ark-' . $name . '.min', $displayData->plugin . 'js' . ARKMEDIA_DS, false, false );

// Load basic stack data
HelperStack::load( false, false, false );

// Grab relevant stack for path data
$stackdata = HelperStack::get( $stack );

// Harvest Field IDs
foreach( $fields->id as $field )
{
	$parents 	= $field->xpath( 'ancestor::fields' );
	$group 		= ( count( $parents ) ) ? current( $parents ) : null;
	$groupname	= ( $group ) ? (string)$group['name'] : null;
	$jfield		= $form->getField( (string)$field['name'], $groupname );
	$id			= '#' . $jfield->id;
	$ids[]		= $id;
	// Get XML Set Path & Pair to Field With ID
	// @note	We need to prefix the path with the base folder like the media field does
	$paths[$id]	= ( $jfield->directory ) ? $stackdata->path . $stackdata->segments->glue . $jfield->directory : '';
}//end foreach

// Remove duplicate entries (e.g. when in Ark Menu)
$ids 			= array_unique( $ids );
$paths 			= array_unique( $paths );

// Build Media URL
$url = array(
	'option' 		=> ARKMEDIA_COMPONENT,
	'stack' 		=> $stack,
	'editor' 		=> $name,
	'path' 			=> '%s',
	'editorname' 	=> '%s',
	'edit' 			=> '%s',
	'editquick' 	=> $params->get( 'edit-quick', false ),
	'editsettings' 	=> $settings,
	'language' 		=> JFactory::getLanguage()->getTag(),
	'callback' 		=> $callback,
	'tmpl' 			=> 'component'
);

// Build Window Options
$parameters = array(
	'location' 		=> 'no',
	'menubar' 		=> 'no',
	'toolbar' 		=> 'no',
	'dependent' 	=> 'yes',
	'minimizable' 	=> 'no',
	'modal' 		=> 'yes',
	'alwaysRaised' 	=> 'yes',
	'resizable' 	=> 'yes',
	'scrollbars' 	=> 'yes'
);

// Set Window Dimensions
switch( Helper::params( 'window-dimensions' ) )
{
	default :
	case 'auto' :
		// Leave Unset For Browser Calculation
		break;

	case 'calc' :
		$parameters['fullscreen'] 	= 'yes';
		$parameters['width'] 		= '\' + screen.width + \'';
		$parameters['height']		= '\' + screen.height + \'';
		break;

	case 'custom' :
		$x = (int)Helper::params( 'window-x' );
		$y = (int)Helper::params( 'window-y' );

		if( $x && $y )
		{
			$parameters['width'] 	= $x;
			$parameters['height']	= $y;
		}//end if
		break;
}//end switch

// Collapse to URL Query String (ensure sprintf's aren't encoded)
$url = JRoute::_( 'index.php?' . JURI::buildQuery( $url ), false );

// Collapse Window String
$parameters = JArrayHelper::toString( $parameters, '=', ',' );

// Strip Double Quotes Added By JString & Convert Singles to JS Doubles
$parameters = ( str_replace( array( '"', '\'' ), array( '', '"' ), $parameters ) );

// Build location to modal triggers (earlier J versions use Mootools)
$container	= '[data-modal]'; // JQuery version only (not SqueezeBox/Mootools)
$control	= '.' . Helper::css( 'joomla.modal' ) . ', .button-select';
?>
<script>
	jQuery( document ).ready( function( $ )
	{
		jQuery.fn.<?php echo $name; ?>(
		{
			css			: {
							root			: 'body',
							ids				: <?php echo json_encode( $ids ); ?>,
							classfield		: '.ark-mediafield',
							datafield		: '[data-mediafield]',
							popup			: '<?php echo ARKMEDIA_COMPONENT_ID; ?>',
							control			: '<?php echo $control; ?>',
							container		: '<?php echo $container; ?>'
						  },
			html 		: {
							url				: '<?php echo $url; ?>',
							callback		: '<?php echo $callback; ?>',
							parameters		: '<?php echo $parameters; ?>'
						  },
			query		: false,
			paths		: <?php echo json_encode( $paths ); ?>
		});
	});
</script>