<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

use Ark\Media\Helper;

defined( '_JEXEC' ) or die();

class plgContentMediaField extends JPlugin
{
	/**
	 * @var		object	Application object
	 */
	public $app;

	/**
	 * @var		string	Ark Extension Name.
	 */
	protected $extension 	= 'com_arkmedia';

	/**
	 * @var		string	The Stack to Restrict the Manager.
	 */
	protected $stack		= 'images';

	/**
	 * @var		string	Plugin Name (for namespacing, config retrieval & more!).
	 */
	protected $name;

	/**
	 * @var		string	The Root of the Joomla Installation Including Trailing DS.
	 */
	protected $base;

	/**
	 * @var		string	The Relative Path to this Plugin
	 */
	protected $plugin;

	/**
	 * Constructor
	 */
	public function __construct( &$subject, $config )
	{
		parent::__construct( $subject, $config );
	}//end function

	/**
	 * Setup Plugin Data Once the Ark is Loaded.
	 *
	 * @return	void
	 */
	public function setup()
	{
		$this->name 	= $this->_name;
		$this->base 	= ARKMEDIA_ROOT . ARKMEDIA_DS;
		$this->plugin 	= str_replace( $this->base, '', ARKMEDIA_ROOT . ARKMEDIA_DS . 'plugins' . ARKMEDIA_DS . $this->_type . ARKMEDIA_DS . $this->name . ARKMEDIA_DS );
	}//end function

	/**
	 * Highjack Content Forms That Include the JForm Field of Media Type
	 * In Order to Include the Ark Media Manager Instead of the Default Media Manager.
	 *
	 * @note	The following methods invoke Ark Media's media field conversion.
	 * 				1. All type="media" fields.
	 * 				2. Add a data-mediafield attribute.
	 * 				3. Add a class="ark-mediafield" attribute.
	 * 				4. Manually trigger a JS event on the element: $( 'input' ).trigger( 'mediafield:request:add' );.
	 *
	 * @param	object	$form	The JForm Object to Manipulate.
	 * @param	string	$data	The Associated Form Data.
	 *
	 * @return	bool	Success Indicator.
	 */
	public function onContentPrepareForm( $form, $data )
	{
		$option 	= $this->app->input->get( 'option', false, 'cmd' );
		$id 		= $this->app->input->get( 'id', false, 'int' );
		$eid 		= $this->app->input->get( 'extension_id', false, 'int' );
		$components = $this->params->get( 'excluded-components', array() );
		$modules 	= $this->params->get( 'excluded-modules', array() );
		$plugins 	= $this->params->get( 'excluded-plugins', array() );
		$templates 	= $this->params->get( 'excluded-templates', array() );

		// Is this field allowed to load in the current extension?
		switch( $option )
		{
			// Bail out for ajax calls to prevent harming our wonderful Ark Menu Extension
			case 'com_ajax' :
				return true;
			
			case 'com_modules' :
				// Existing or new module? Filter table logic based on ID or EID
				$eid 	= $this->app->getUserState( 'com_modules.add.module.extension_id' );
				$table 	= JTable::getInstance( ( $id ? 'Module' : 'Extension' ), 'JTable' );

				if( $table->load( ( $id ? $id : $eid ) ) && in_array( ( $id ? $table->module : $table->element ), $modules ) )
				{
					return true;
				}//end if
				break;

			case 'com_plugins' :
				$table = JTable::getInstance( 'Extension', 'JTable' );

				if( $table->load( $eid ) && in_array( $table->element, $plugins ) )
				{
					return true;
				}//end if
				break;
			
			case 'com_templates' :
				$table = JTable::getInstance( 'Style', 'TemplatesTable' );

				if( $table->load( $id ) && in_array( $table->template, $templates ) )
				{
					return true;
				}//end if
				break;

			// Filter components
			default :
				if( in_array( $option, $components ) )
				{
					return true;
				}//end if
				break;
		}//end switch

		// Required in Some Instances
		jimport( 'joomla.filesystem.folder' );
		jimport( 'joomla.filesystem.file' );

		if( !( $form instanceof JForm ) )
		{
			return false;
		}//end if

		$xml = $form->getXML();

		if( $xml && ( $xml instanceof SimpleXMLElement ) )
		{
			$loader		= JPATH_ADMINISTRATOR . chr( 47 ) . 'components' . chr( 47 ) . $this->extension . chr( 47 ) . 'initiate.php';
			$idfields	= $xml->xpath( '//field[@type="media"]' );
			$classfield = $xml->xpath( '//field[@class="ark-mediafield"]' );
			$datafield 	= $xml->xpath( '//field[@data-mediafield]' );

			// Add Our Interception JS/Layout if we Find a Media Field in the Form
			if( count( $idfields ) || count( $classfield ) || count( $datafield ) )
			{
				// Initiate Ark
				if( JFile::exists( $loader ) )
				{
					defined( '_ARKMEDIA_EXEC' ) or define( '_ARKMEDIA_EXEC', true );

					require_once $loader;

					$this->setup();
				}
				else
				{
					return;
				}//end if

				$layouts = $this->base . $this->plugin . 'layouts';
				$options = (object)array( 
					'name' 		=> $this->name, 
					'stack' 	=> $this->params->get( 'folder-default', $this->stack ), 
					'plugin' 	=> $this->plugin, 
					'params' 	=> $this->params, 
					'settings' 	=> false, // Settings is currently never appropriate for this plugin
					'form' 		=> $form, 
					'fields' 	=> (object)array( 
						'id' 	=> $idfields,
						'class' => $classfield,
						'data' 	=> $datafield
					)
				);

				echo Helper::layout( 'js.' . $this->name, $options, array( 'base' => $layouts, 'client' => 'auto' ) );
			}//end if
		}//end if

		return true;
	}//end function
}//end class