<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

defined( '_JEXEC' ) or die;

class com_arkmediaInstallerScript
{
	/**
	 * @var		object	Extension JTable instance.
	 */
	protected $table 		= null;

	/**
	 * @var		string	The current version from the manifest cache (before this new version is installed)
	 */
	protected $version 		= null;

	/**
	 * Method to Install the Package.
	 *
	 * @note 	These Custom Attributes Will Wipe Any Previously Stored Params in the DB.
	 *
	 * @param	class	$adapter	The Object Responsible for Running this Script.
	 */
	public function install( JAdapterInstance $adapter )
	{

	}//end function

	/**
	 * Method to Uninstall the Package.
	 *
	 * @param	class	$adapter	The Object Responsible for Running this Script.
	 */
	public function uninstall( JAdapterInstance $adapter )
	{

	}//end function

	/**
	 * Method to Update the Package.
	 *
	 * @note	$this->table contains the old instance & manifest. Re-query for new version.
	 *
	 * @note	Use (string)current( $adapter->getManifest()->xpath( '//version' ) ); to access new version.
	 *
	 * @param	class	$adapter	The Object Responsible for Running this Script.
	 */
	public function update( JAdapterInstance $adapter )
	{
		// Check record was loaded
		if( $this->table )
		{
			// Convert file type parameters if an old version in main stack plugin defaults
			if( version_compare( $this->version, '2.1.0', '<=' ) )
			{
				$dbo = JFactory::getDBO();
				$sql = $dbo->getQuery( true );
				$sql->select( array( $dbo->qn( 'extension_id' ), $dbo->qn( 'element' ), $dbo->qn( 'params' ) ) )
					->from( '#__extensions' )
					->where( $dbo->qn( 'type' ) . ' = ' . $dbo->q( 'plugin' ) )
					->where( $dbo->qn( 'folder')  . ' = ' . $dbo->q( str_replace( 'com_', '', $adapter->getName() ) ) );
				$plugins = $dbo->setQuery( $sql )->loadObjectList();

				foreach( $plugins as $plugin )
				{
					$params 	= new JRegistry( $plugin->params );
					$updated	= true;

					switch( $plugin->element )
					{
						case 'images' :
							$params->set( 'allowed-types', array( '[image/jpg,image/jpeg][jpg,jpeg]', '[image/png][png]', '[image/gif][gif]', '[image/bmp][bmp]', '[image/svg+xml][svg]', '[image/x-icon][ico]', '[image/tiff][tif,tiff]' ) );
							break;

						case 'audios' :
							$params->set( 'allowed-types', array( '[audio/mpeg][mp3]', '[audio/x-ms-wma][aac]', '[audio/ogg][ogg]', '[audio/x-ms-wma][wma]', '[audio/x-wav][wav]', '[audio/x-aiff][aiff]', '[audio/mp4][m4a,mp4a]', '[audio/x-flac][flac]', '[audio/midi][mid]' ) );
							break;

						case 'videos' :
							$params->set( 'allowed-types', array( '[application/x-shockwave-flash][swf]', '[video/x-flv][flv]', '[video/x-msvideo][avi]', '[video/mp4][mp4]', '[video/x-m4v][m4v]', '[video/x-ms-wmv][wmv]', '[video/quicktime][mov]', '[video/mpeg][mpg,mpeg]', '[video/x-ms-asf][asf]' ) );
							break;

						case 'documents' :
							$params->set( 'allowed-types', array( '[text/plain][txt]', '[text/rtf][rtf]', '[application/zip,application/x-zip][zip,gzip,zipx]', '[application/x-7z-compressed][7z,s7z]', '[application/x-tar][tar,tar.gz,tgz,tar.Z,tar.bz2,tlz]', '[application/x-rar,application/x-rar-compressed][rar]', '[application/pdf][pdf]', '[application/x-shockwave-flash][swf]', '[application/x-shockwave-flash][fla]', '[application/msword,application/vnd.openxmlformats-officedocument.wordprocessingml.document][doc,docx]', '[application/excel,application/vnd.ms-excel,application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,text/csv][xls,xlsx,csv]', '[application/powerpoint,application/vnd.ms-powerpoint,application/vnd.openxmlformats-officedocument.presentationml.presentation][ppt,pptx]', '[application/vnd.oasis.opendocument.presentation][odp]', '[application/vnd.oasis.opendocument.text][odt]', '[application/vnd.oasis.opendocument.spreadsheet][ods]', '[application/vnd.oasis.opendocument.graphics][odg]', '[text/html][html]', '[application/xhtml+xml][xhtml]', '[application/xml][xml]', '[application/x-font-ttf][ttf]', '[application/x-font-woff][woff]', '[application/x-font-otf][otf]', '[application/vnd.ms-fontobject][eot]' ) );
							break;

						default :
							$updated = false;
							break;
					}//end switch

					if( $updated )
					{
						$sql = $dbo->getQuery( true );
						$sql->update( '#__extensions' )
							->set( $dbo->qn( 'params' ) . ' = ' . $dbo->q( $params->toString() ) )
							->where( $dbo->qn( 'extension_id' ) . ' = ' . (int)$plugin->extension_id );
						$dbo->setQuery( $sql )-> execute();
					}//end if
				}//end foreach
			}//end if

			// Convert file type parameters if an old version
			if( version_compare( $this->version, '2.2.1', '<=' ) )
			{
				$params 	= new JRegistry( $this->table->params );
				$oldtypes	= $params->get( 'allowed-types', null );
				$newtypes	= new stdClass;

				// Check params have been saved & aren't blank (casting null still counts: 0)
				if( count( (array)$oldtypes ) )
				{
					// Loop stack file types & convert to new syntax
					foreach( $oldtypes as $stack => $oldtypes )
					{
						// Set up new param list
						$newtypes->{ $stack } = array();

						// Force override from MIME,[EXT1,EXT2] to [MIME][EXT1,EXT2]
						// Also broken versions like [[MIME][MIMEEXT1,EXT2] && [[MIME][EXT1]][] etc.
						// (enclosing parentheses are now required for mime & ext to allow for multiple options)
						foreach( $oldtypes as $val )
						{
							// Images
							if( strpos( $val, 'image/jpg' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/jpg,image/jpeg][jpg,jpeg]';
							}
							else if( strpos( $val, 'image/png' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/png][png]';
							}
							else if( strpos( $val, 'image/gif' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/gif][gif]';
							}
							else if( strpos( $val, 'image/bmp' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/bmp][bmp]';
							}
							else if( strpos( $val, 'image/svg+xml' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/svg+xml][svg]';
							}
							else if( strpos( $val, 'image/x-icon' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/x-icon][ico]';
							}
							else if( strpos( $val, 'image/tiff' ) !== false )
							{
								$newtypes->{ $stack }[] = '[image/tiff][tif,tiff]';
							}
							// Audios
							else if( strpos( $val, 'audio/mpeg' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/mpeg][mp3]';
							}
							else if( strpos( $val, 'audio/x-ms-wma' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/x-ms-wma][aac]';
								$newtypes->{ $stack }[] = '[audio/x-ms-wma][wma]';
							}
							else if( strpos( $val, 'audio/ogg' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/ogg][ogg]';
							}
							else if( strpos( $val, 'audio/x-wav' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/x-wav][wav]';
							}
							else if( strpos( $val, 'audio/x-aiff' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/x-aiff][aiff]';
							}
							else if( strpos( $val, 'audio/mp4' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/mp4][m4a,mp4a]';
							}
							else if( strpos( $val, 'audio/x-flac' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/x-flac][flac]';
							}
							else if( strpos( $val, 'audio/midi' ) !== false )
							{
								$newtypes->{ $stack }[] = '[audio/midi][mid]';
							}
							// Videos
							else if( strpos( $val, 'application/x-shockwave-flash' ) !== false )
							{
								// Also in Documents
								$newtypes->{ $stack }[] = '[application/x-shockwave-flash][swf]';
								$newtypes->{ $stack }[] = '[application/x-shockwave-flash][fla]';
							}
							else if( strpos( $val, 'video/x-flv' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/x-flv][flv]';
							}
							else if( strpos( $val, 'video/x-msvideo' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/x-msvideo][avi]';
							}
							else if( strpos( $val, 'video/mp4' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/mp4][mp4]';
							}
							else if( strpos( $val, 'video/x-m4v' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/x-m4v][m4v]';
							}
							else if( strpos( $val, 'video/x-ms-wmv' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/x-ms-wmv][wmv]';
							}
							else if( strpos( $val, 'video/quicktime' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/quicktime][mov]';
							}
							else if( strpos( $val, 'video/mpeg' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/mpeg][mpg,mpeg]';
							}
							else if( strpos( $val, 'video/x-ms-asf' ) !== false )
							{
								$newtypes->{ $stack }[] = '[video/x-ms-asf][asf]';
							}
							// Documents
							else if( strpos( $val, 'text/plain' ) !== false )
							{
								$newtypes->{ $stack }[] = '[text/plain][txt]';
							}
							else if( strpos( $val, 'text/rtf' ) !== false )
							{
								$newtypes->{ $stack }[] = '[text/rtf][rtf]';
							}
							else if( strpos( $val, 'application/zip' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/zip,application/x-zip][zip,gzip,zipx]';
							}
							else if( strpos( $val, 'application/x-zip' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/zip,application/x-zip][zip,gzip,zipx]';
							}
							else if( strpos( $val, 'application/x-7z-compressed' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-7z-compressed][7z,s7z]';
							}
							else if( strpos( $val, 'application/x-tar' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-tar][tar,tar.gz,tgz,tar.Z,tar.bz2,tlz]';
							}
							else if( strpos( $val, 'application/x-rar' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-rar,application/x-rar-compressed][rar]';
							}
							else if( strpos( $val, 'application/x-rar-compressed' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-rar,application/x-rar-compressed][rar]';
							}
							else if( strpos( $val, 'application/pdf' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/pdf][pdf]';
							}
							else if( strpos( $val, 'application/msword' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/msword,application/vnd.openxmlformats-officedocument.wordprocessingml.document][doc,docx]';
							}
							else if( strpos( $val, 'application/vnd.openxmlformats-officedocument.wordprocessingml.document' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/msword,application/vnd.openxmlformats-officedocument.wordprocessingml.document][doc,docx]';
							}
							else if( strpos( $val, 'application/excel' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/excel,application/vnd.ms-excel,application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,text/csv][xls,xlsx,csv]';
							}
							else if( strpos( $val, 'application/vnd.ms-excel' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/excel,application/vnd.ms-excel,application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,text/csv][xls,xlsx,csv]';
							}
							else if( strpos( $val, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,text/csv' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/excel,application/vnd.ms-excel,application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,text/csv][xls,xlsx,csv]';
							}
							else if( strpos( $val, 'application/powerpoint' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/powerpoint,application/vnd.ms-powerpoint,application/vnd.openxmlformats-officedocument.presentationml.presentation][ppt,pptx]';
							}
							else if( strpos( $val, 'application/vnd.ms-powerpoint' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/powerpoint,application/vnd.ms-powerpoint,application/vnd.openxmlformats-officedocument.presentationml.presentation][ppt,pptx]';
							}
							else if( strpos( $val, 'application/vnd.openxmlformats-officedocument.presentationml.presentation' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/powerpoint,application/vnd.ms-powerpoint,application/vnd.openxmlformats-officedocument.presentationml.presentation][ppt,pptx]';
							}
							else if( strpos( $val, 'application/vnd.oasis.opendocument.presentation' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/vnd.oasis.opendocument.presentation][odp]';
							}
							else if( strpos( $val, 'application/vnd.oasis.opendocument.text' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/vnd.oasis.opendocument.text][odt]';
							}
							else if( strpos( $val, 'application/vnd.oasis.opendocument.spreadsheet' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/vnd.oasis.opendocument.spreadsheet][ods]';
							}
							else if( strpos( $val, 'application/vnd.oasis.opendocument.graphics' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/vnd.oasis.opendocument.graphics][odg]';
							}
							else if( strpos( $val, 'text/html' ) !== false )
							{
								$newtypes->{ $stack }[] = '[text/html][html]';
							}
							else if( strpos( $val, 'application/xhtml+xml' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/xhtml+xml][xhtml]';
							}
							else if( strpos( $val, 'application/xml' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/xml][xml]';
							}
							else if( strpos( $val, 'application/x-font-ttf' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-font-ttf][ttf]';
							}
							else if( strpos( $val, 'application/x-font-woff' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-font-woff][woff]';
							}
							else if( strpos( $val, 'application/x-font-otf' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/x-font-otf][otf]';
							}
							else if( strpos( $val, 'application/vnd.ms-fontobject' ) !== false )
							{
								$newtypes->{ $stack }[] = '[application/vnd.ms-fontobject][eot]';
							}//end if
						}//end foreach
					}//end foreach

					// Set params back
					$params->set( 'allowed-types', $newtypes );

					$this->table->params = $params->toString();

					// Store changes
					$this->table->store();
				}//end if
			}//end if
		}//end if
	}//end function

	/**
	 * Method to Run Before an install/update/uninstall Method.
	 *
	 * @param	string	$route		Which action is happening (install || uninstall || discover_install).
	 * @param	class	$adapter	The Object Responsible for Running this Script.
	 */
	public function preflight( $route, JAdapterInstance $adapter )
	{
		// Load & store the old version of the component
		$this->table = JTable::getInstance( 'extension' );
		$componentid = $this->table->find( array( 'type' => 'component', 'element' => $adapter->getName() ) );

		// Load Record by Extension Name
		if( $componentid && $this->table->load( $componentid ) )
		{
			$manifest 		= json_decode( $this->table->manifest_cache );
			$this->version	= $manifest->version;
		}
		else
		{
			// Clear table instance
			$this->table	= null;
		}//end if
	}//end function

	/**
	 * Method to Run After an install/update/uninstall Method.
	 *
	 * @param	string	$route		Which action is happening (install || uninstall || discover_install).
	 * @param	class	$adapter	The Object Responsible for Running this Script.
	 */
	public function postflight( $route, JAdapterInstance $adapter )
	{
		// Fresh Install Only (to save destroying existing permissions)
		if( $route === 'install' )
		{
			$app 	= JFactory::getApplication();
			$asset 	= JTable::getInstance( 'Asset' );
			$class 	= str_replace( 'InstallerScript', '', __CLASS__ );
			$path 	= JPATH_ROOT . chr( 47 ) . 'administrator' . chr( 47 ) . 'components' . chr( 47 ) . $class . chr( 47 );

			// Get Asset Entry
			if( $asset->loadByName( $class ) )
			{
				// Load Our Access File
				$xml = simplexml_load_file( $path . 'access.xml' );

				if( $xml instanceof SimpleXMLElement )
				{
					// Grab Entries With Defaults
					$actions 	= $xml->xpath( '//action[@name][@default]' );
					$rules 		= new stdClass;

					// Add Action Entry & Assign Defaults to the Entry
					foreach( $actions as $action )
					{
						// Defaults Are CSV (Assign to '1' to Signify Active Permission)
						$defaults = explode( ',', (string)$action['default'] );
						$rules->{ (string)$action['name'] } = array_fill_keys( $defaults, 1 );
					}//end foreach

					// Set & Store Default Permissions
					$asset->rules = json_encode( $rules );

					if( !$asset->store() )
					{
						$app->enQueueMessage( JText::_( JString::strtoupper( $class ) . '_MSG_PERMISSION_INSTALL_FAIL' ), 'warning' );
					}//end if
				}
				else
				{
					$app->enQueueMessage( JText::_( JString::strtoupper( $class ) . '_MSG_PERMISSION_INSTALL_FAIL' ), 'warning' );
				}//end if
			}
			else
			{
				$app->enQueueMessage( JText::_( JString::strtoupper( $class ) . '_MSG_PERMISSION_INSTALL_FAIL' ), 'warning' );
			}//end if
		}//end if
	}//end function
}//end class