<?php
/**
 * @version     2.4.1
 * @package     com_arkmedia
 * @copyright   Copyright (C) 2018. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      WebxSolution Ltd - http://www.arkextensions.com
 */

use Ark\Media\Helper, Ark\Media\HelperStack, Ark\Media\HelperFileSystem;

defined( '_JEXEC' ) or die();

class plgArkMediaFeatureUpload extends JPlugin
{
	/**
	 * @var		string	Plugin Name (for namespacing, config retrieval & more!)
	 */
	protected $name = 'upload';

	/**
	 * @var		string	The Root of the Joomla Installation Including Trailing DS
	 */
	protected $base;

	/**
	 * @var		string	The Relative Path to this Plugin
	 */
	protected $plugin;

	/**
	 * @var		string	Base for JText
	 */
	protected $jtext;

	/**
	 * Constructor
	 */
	public function __construct( &$subject, $config )
	{
		// Load Language for Title
		JFactory::getLanguage()->load( 'plg_' . ARKMEDIA_PLUGIN_FEATURE . '_' . $this->name, JPATH_ADMINISTRATOR );

		$this->base 	= ARKMEDIA_ROOT . ARKMEDIA_DS;
		$this->plugin 	= str_replace( $this->base, '', ARKMEDIA_PLUGIN_FEATURES . $this->name . ARKMEDIA_DS );
		$this->jtext 	= 'PLG_' . JString::strtoupper( ARKMEDIA_PLUGIN_FEATURE ) . '_' . JString::strtoupper( $this->name ) . '_';

		parent::__construct( $subject, $config );
	}//end function

	/**
	 * Catch All Ark Form Events to Add Additional JForm/Items to the Forms.
	 *
	 * @note 	This Isn't a Stack Plugin so Don't Register Any Namespaces.
	 *
	 * @param	string	$name		The Current Form Name
	 * @param	object	$form		The Form Object to Manipulate
	 * @param	bool	$namespace	Whether to Namespace Each Form Field With the Stack Name (opt in)
	 *
	 * @return	void
	 */
	public function onArkAfterForm( &$name = '', &$form = null, $namespace = false )
	{
		$file = $this->base . $this->plugin . 'forms/' . $name . '.xml';

		// If We Have Our Own Additions Then Add Them
		if( JFile::exists( $file ) )
		{
			// Merge the Two Forms
			$form->loadFile( $file );

			// Dynamically Add Layout Path & Other Data
			$xml = simplexml_load_file( $file );

			if( $xml instanceof SimpleXMLElement )
			{
				// Only Expect/Need One
				$field = current( $xml->xpath( '//field[@path=""]' ) );

				if( $field )
				{
					$form->setFieldAttribute( (string)$field['name'], 'js',  $this->plugin . 'js' );
					$form->setFieldAttribute( (string)$field['name'], 'css',  $this->plugin . 'css' );
					$form->setFieldAttribute( (string)$field['name'], 'path', $this->plugin . 'layouts' );
					$form->setFieldAttribute( (string)$field['name'], 'jtext', $this->jtext );
				}//end if
			}//end if
		}//end if
	}//end function

	/**
	 * Catch Controller Task to Handle Upload Response.
	 *
	 * @param	object	$ctrl		The Controller Object.
	 * @param	mixed	$data		Additional Event Data.
	 *
	 * @method	ajax
	 *
	 * @return	void
	 */
	public function onArkAfterTask( &$ctrl = null, $data = null )
	{
		$app 	= JFactory::getApplication();
		$task 	= $app->input->get( 'task', false, 'cmd' );

		// Catch Our Task
		if( $task === 'file' )
		{
			// Gather Processed Data
			$form 		= $app->input->get( 'jform', array(), 'array' );
			$files 		= $app->input->files->get( 'jform', array(), 'array' );
			$model		= $ctrl->getModel( 'list' );
			$file 		= $data;

			// Collect & Process Queue
			$queue 		= $app->getMessageQueue();
			$error		= null;
			$messages 	= JArrayHelper::getColumn( $queue, 'message' );
			$item		= null;

			// Grab First Error From Queue
			foreach( $queue as $message )
			{
				if( in_array( $message['type'], array( 'error', 'warning', 'notice' ) ) )
				{
					$error = $message['message'];

					break;
				}//end if
			}//end foreach

			// Object Data for Adding to the Observer
			// @note	The New Name Not the Old One
			try
			{
				$item 		= $model->item( $form['stack'], $form['path'], $file, 'file' ); 
			}
			catch( Exception $e )
			{
				// Don't Flag as a Full Error as the Upload was Largely Successful
				$messages[]	= JText::_( $this->jtext . 'MSG_LOAD_FAIL' );
			}//end try

			// Handle item data as item maybe empty (on error)
			$icon			= ( $item ) ? $item->preview['icon'] : '';
			$thumbnail		= ( $item ) ? $item->preview['thumbnail'] : '';

			// Set Up Return Object
			$observer		= array( 'stack' => $form['stack'], 'path' => $form['path'] );
			$result 		= (object)array( 'files' => array() );
			$result->files[] = (object)array( 
				'name' 		=> $files['file']['name'], // Original Name for Matching Back to Element
				'size' 		=> $files['file']['size'], // File Size for Re-Display
				'type' 		=> $files['file']['type'], // File Mime Type
				'error' 	=> $error, // 1st Error (only needed to detect the presence of errors)
				'messages' 	=> $messages, // All Messages (including errors)
				'preview' 	=> (object)array( 'thumbnail' => $thumbnail, 'icon' => $icon ),
				'observer'	=> (object)array_merge( ( $item ? (array)$item : array() ), $observer ) // $item may be false
			);

			echo json_encode( $result );

			jexit();
		}//end if
	}//end function
}//end class